@@ -61,6 +61,7 @@ class AquaDeployment(DataClassSerializable):
     id: str = None
     display_name: str = None
     aqua_service_model: bool = None
+    aqua_model_name: str = None
     state: str = None
     description: str = None
     created_on: str = None
@@ -116,13 +117,17 @@ class AquaDeployment(DataClassSerializable):
             ),
         )
 
+        freeform_tags = oci_model_deployment.freeform_tags or UNKNOWN_DICT
+        aqua_service_model_tag = freeform_tags.get(
+            Tags.AQUA_SERVICE_MODEL_TAG.value, None
+        )
+        aqua_model_name = freeform_tags.get(Tags.AQUA_MODEL_NAME_TAG.value, UNKNOWN)
+
         return AquaDeployment(
             id=oci_model_deployment.id,
             display_name=oci_model_deployment.display_name,
-            aqua_service_model=oci_model_deployment.freeform_tags.get(
-                Tags.AQUA_SERVICE_MODEL_TAG.value
-            )
-            is not None,
+            aqua_service_model=aqua_service_model_tag is not None,
+            aqua_model_name=aqua_model_name,
             shape_info=shape_info,
             state=oci_model_deployment.lifecycle_state,
             lifecycle_details=getattr(
@@ -137,7 +142,7 @@ class AquaDeployment(DataClassSerializable):
                 ocid=oci_model_deployment.id,
                 region=region,
             ),
-            tags=oci_model_deployment.freeform_tags,
+            tags=freeform_tags,
         )
 
 
@@ -235,17 +240,9 @@ class AquaDeploymentApp(AquaApp):
             An Aqua deployment instance
 
         """
-        # todo: revisit error handling and pull deployment image info from config
-        # if not AQUA_MODEL_DEPLOYMENT_IMAGE:
-        #     raise AquaValueError(
-        #         f"AQUA_MODEL_DEPLOYMENT_IMAGE must be available in environment variables to "
-        #         f"continue with Aqua model deployment."
-        #     )
-
-        # todo: for fine tuned models, skip model creation.
         # Create a model catalog entry in the user compartment
         aqua_model = AquaModelApp().create(
-            model_id=model_id, comparment_id=compartment_id, project_id=project_id
+            model_id=model_id, compartment_id=compartment_id, project_id=project_id
         )
 
         tags = {}
@@ -257,6 +254,8 @@ class AquaDeploymentApp(AquaApp):
             if tag in aqua_model.freeform_tags:
                 tags[tag] = aqua_model.freeform_tags[tag]
 
+        tags.update({Tags.AQUA_MODEL_NAME_TAG.value: aqua_model.display_name})
+
         # Set up info to get deployment config
         config_source_id = model_id
         model_name = aqua_model.display_name
@@ -535,7 +534,7 @@ class AquaDeploymentApp(AquaApp):
             log_group_id=log_group_id,
             log_id=log_id,
             compartment_id=model_deployment.compartment_id,
-            source_id=model_deployment.id
+            source_id=model_deployment.id,
         )
 
         return AquaDeploymentDetail(
