@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8; -*-
 
-# Copyright (c) 2020, 2022 Oracle and/or its affiliates.
+# Copyright (c) 2020, 2023 Oracle and/or its affiliates.
 # Licensed under the Universal Permissive License v 1.0 as shown at https://oss.oracle.com/licenses/upl/
 
 from __future__ import print_function, absolute_import
@@ -131,7 +131,6 @@ class RecommendationTransformer(TransformerMixin):
         self.feature_metadata_[self.target_] = self.target_type_
 
         for column in df.columns.values[df.isnull().any()]:
-
             # filter out columns that were discovered as constant or primary key columns in the previous step,
             # as they would get dropped before imputation
             if (
@@ -246,21 +245,22 @@ class RecommendationTransformer(TransformerMixin):
             if not self.is_balanced and self.fix_imbalance:
                 target_value_counts = df[self.target_].value_counts()
                 minority_class_len = min(
-                    target_value_counts.iteritems(), key=lambda k: k[1]
+                    target_value_counts.items(), key=lambda k: k[1]
                 )[1]
                 majority_class_len = max(
-                    target_value_counts.iteritems(), key=lambda k: k[1]
+                    target_value_counts.items(), key=lambda k: k[1]
                 )[1]
                 minor_majority_ratio = minority_class_len / majority_class_len
 
-                # Suggest down sampling if minor_majority_ratio is 1:1000
-                suggested_sampling = (
-                    "Do nothing"
-                    if len(df) <= utils.MAX_LEN_FOR_UP_SAMPLING
-                    else "Down-sample"
-                    if minor_majority_ratio >= utils.MIN_RATIO_FOR_DOWN_SAMPLING
-                    else "Do nothing"
-                )
+                # up-sample if length of dataframe is less than or equal to MAX_LEN_FOR_UP_SAMPLING = 5000
+                # down-sample if minor_majority_ratio is greater than or equal to MIN_RATIO_FOR_DOWN_SAMPLING = 1/20
+                if len(df) <= utils.MAX_LEN_FOR_UP_SAMPLING:
+                    suggested_sampling = "Up-sample"
+                elif minor_majority_ratio >= utils.MIN_RATIO_FOR_DOWN_SAMPLING:
+                    suggested_sampling = "Down-sample"
+                else:
+                    suggested_sampling = "Do nothing"
+
                 self._build_recommendation(
                     recommendations,
                     "fix_imbalance",
