@@ -1,14 +1,15 @@
 #!/usr/bin/env python
 # -*- coding: utf-8; -*-
 
-# Copyright (c) 2020, 2022 Oracle and/or its affiliates.
+# Copyright (c) 2020, 2023 Oracle and/or its affiliates.
 # Licensed under the Universal Permissive License v 1.0 as shown at https://oss.oracle.com/licenses/upl/
 
 import numpy as np
 import pandas as pd
-from ads.common import utils
+from ads.common import logger
 from ads.explanations.base_explainer import LocalExplainer
 from ads.explanations.mlx_interface import init_lime_explainer
+from ads.common.decorator.deprecate import deprecated
 
 
 class MLXLocalExplainer(LocalExplainer):
@@ -23,6 +24,10 @@ class MLXLocalExplainer(LocalExplainer):
         - Regression
     """
 
+    @deprecated(
+        details="Working with AutoML has moved from within ADS to working directly with the AutoMLx library. AutoMLx are preinstalled in conda pack automlx_p38_cpu_v2 and later, and can now be updated independently of ADS. AutoMLx documentation may be found at https://docs.oracle.com/en-us/iaas/tools/automlx/latest/html/multiversion/v23.1.1/index.html. Notebook examples are in Oracle's samples repository: https://github.com/oracle-samples/oci-data-science-ai-samples/tree/master/notebook_examples and a migration tutorial can be found at https://accelerated-data-science.readthedocs.io/en/latest/user_guide/model_training/automl/quick_start.html .",
+        raise_error=True,
+    )
     def __init__(self):
         super(LocalExplainer, self).__init__()
         self.explainer = None
@@ -34,6 +39,7 @@ class MLXLocalExplainer(LocalExplainer):
         num_features=None,
         report_fidelity=True,
         validation_percentiles=None,
+        selected_features=None,
     ):
         """
         Explains the sample, row. Returns a local explanation object.
@@ -60,14 +66,17 @@ class MLXLocalExplainer(LocalExplainer):
             1 is very local to the sample to explain, the evaluation dataset at percentile 10 evaluates
             the explanation quality further away from the instance to explain (row). This can be helpful
             to see how the explanation generalizes to nearby samples. By default None ([1, 5, 15]).
+        selected_features: list[int], optional
+            List of the selected features (list of numbers of features). It can be any subset of
+            the original features that are in the dataset provided to the model.
+            Default value is None.
 
         Return
         ------
         LocalExplanation
             Local explanation object.
         """
-        if self.explainer is None:
-            self._init_explainer()
+        self.configure_local_explainer(selected_features=selected_features)
         assert row is not None and isinstance(row, pd.DataFrame) and row.shape[0] == 1
 
         labels = list(range(len(self.class_names)))
@@ -75,15 +84,26 @@ class MLXLocalExplainer(LocalExplainer):
         if num_features is None:
             num_features = len(row.columns.values)
 
-        explanation = self.explainer.compute(
-            row.copy(),
-            y=None if y is None else y.copy(),
-            labels=labels,
-            verbose=False,
-            num_features=num_features,
-            report_fidelity=report_fidelity,
-            validation_percentiles=validation_percentiles,
-        )[0]
+        try:
+            explanation = self.explainer.compute(
+                row.copy(),
+                y=None if y is None else y.copy(),
+                labels=labels,
+                verbose=False,
+                num_features=num_features,
+                report_fidelity=report_fidelity,
+                validation_percentiles=validation_percentiles,
+            )[0]
+        except IndexError as e:
+            if selected_features is not None:
+                raise IndexError(
+                    f"Unable to generate local explanations due to: {e}. "
+                    f"selected_features must be a list of features within the bounds of the existing features "
+                    f"(that were provided to model). Provided selected_features: {selected_features}."
+                )
+        except Exception as e:
+            logger.error(f"Unable to generate local explanations due to: {e}.")
+            raise e
 
         return LocalExplanation(explanation, self.class_names)
 
@@ -93,13 +113,14 @@ class MLXLocalExplainer(LocalExplainer):
         explainer with the provided configuration parameters.
 
         Supported configuration options in kwargs:
-            - **surrogate_model** (str): Surrogate model to use. Can be 'linear' or 'decision_tree'.
-            - **num_samples** (int): Number of generated samples to fit the surrogate model.
-            - **exp_sorting** (str): Feature importance sorting. Can be 'absolute' or 'ordered'.
-            - **scale_weight** (bool): Normalizes the feature importance coefficients from LIME to sum to one.
-            - **client**: Only allowed to be None to disable parallelization.
-            - **batch_size** (int): Number of local explanations per Dask worker.
-            - **random_state** (`None` or `int` or instance of `RandomState`): the random state
+            - `surrogate_model` (str): Surrogate model to use. Can be 'linear' or 'decision_tree'.
+            - `num_samples` (int): Number of generated samples to fit the surrogate model.
+            - `exp_sorting` (str): Feature importance sorting. Can be 'absolute' or 'ordered'.
+            - `scale_weight` (bool): Normalizes the feature importance coefficients from LIME to sum to one.
+            - `client`: Only allowed to be None to disable parallelization.
+            - `batch_size` (int): Number of local explanations per Dask worker.
+            - `random_state` (`None` or `int` or instance of `RandomState`): the random state.
+            - `selected_features` (`None`, or `list`): list of the selected features numbers.
 
         Parameters
         ----------
@@ -118,6 +139,7 @@ class MLXLocalExplainer(LocalExplainer):
             "exp_sorting",
             "scale_weight",
             "batch_size",
+            "selected_features",
         ]
         for k, v in kwargs.items():
             if k not in avail_args:
@@ -133,6 +155,12 @@ class MLXLocalExplainer(LocalExplainer):
             raise ValueError(
                 "Invalid surrogate_model provided. Currently only supports linear or decision_tree"
             )
+        selected_features = kwargs.get("selected_features", None)
+        if selected_features is not None and not isinstance(selected_features, list):
+            raise ValueError(
+                f"selected_features ({selected_features}) value must be a list of features, "
+                f"but it is of type: {type(selected_features)}."
+            )
 
         self._init_explainer(**kwargs)
         return self
@@ -147,7 +175,7 @@ class MLXLocalExplainer(LocalExplainer):
             HTML object representing the explainer summary.
         """
         if self.explainer is None:
-            self._init_explainer()
+            self.configure_local_explainer()
         return self.explainer.show_in_notebook()
 
     def _init_explainer(self, **kwargs):
@@ -168,8 +196,7 @@ class MLXLocalExplainer(LocalExplainer):
             self.y_train,
             self.mode,
             class_names=self.class_names,
-            use_pre_selected_features=True,
-            **kwargs
+            **kwargs,
         )
 
 
@@ -244,7 +271,7 @@ class LocalExplanation:
             labels=labels,
             colormap=colormap,
             return_wordcloud=return_wordcloud,
-            **kwargs
+            **kwargs,
         )
 
     def get_diagnostics(self):
