@@ -1,12 +1,13 @@
 #!/usr/bin/env python
 # -*- coding: utf-8; -*-
 
-# Copyright (c) 2021, 2023 Oracle and/or its affiliates.
+# Copyright (c) 2021, 2024 Oracle and/or its affiliates.
 # Licensed under the Universal Permissive License v 1.0 as shown at https://oss.oracle.com/licenses/upl/
 from __future__ import annotations
 
 import datetime
 import logging
+import oci
 import os
 import time
 import traceback
@@ -34,10 +35,20 @@ from ads.jobs.builders.runtimes.artifact import Artifact
 from ads.jobs.builders.runtimes.container_runtime import ContainerRuntime
 from ads.jobs.builders.runtimes.python_runtime import GitPythonRuntime
 
+from ads.common.dsc_file_system import (
+    OCIFileStorage,
+    DSCFileSystemManager,
+    OCIObjectStorage,
+)
+from ads.common.decorator.utils import class_or_instance_method
+
 logger = logging.getLogger(__name__)
 
 SLEEP_INTERVAL = 3
 WAIT_SECONDS_AFTER_FINISHED = 90
+MAXIMUM_MOUNT_COUNT = 5
+FILE_STORAGE_TYPE = "FILE_STORAGE"
+OBJECT_STORAGE_TYPE = "OBJECT_STORAGE"
 
 
 class DSCJob(OCIDataScienceMixin, oci.data_science.models.Job):
@@ -161,10 +172,12 @@ class DSCJob(OCIDataScienceMixin, oci.data_science.models.Job):
             )
             subnet_id = infra.get(
                 "subnetId",
-                nb_config.subnet_id
-                if infra_type
-                != JobInfrastructureConfigurationDetails.JOB_INFRASTRUCTURE_TYPE_ME_STANDALONE
-                else None,
+                (
+                    nb_config.subnet_id
+                    if infra_type
+                    != JobInfrastructureConfigurationDetails.JOB_INFRASTRUCTURE_TYPE_ME_STANDALONE
+                    else None
+                ),
             )
             job_shape_config_details = infra.get("jobShapeConfigDetails", {})
             memory_in_gbs = job_shape_config_details.get(
@@ -257,10 +270,8 @@ class DSCJob(OCIDataScienceMixin, oci.data_science.models.Job):
                 nb_session = None
             if nb_session:
                 nb_config = getattr(
-                    nb_session,
-                    "notebook_session_config_details",
-                    getattr(nb_session, "notebook_session_configuration_details", None),
-                )
+                    nb_session, "notebook_session_config_details", None
+                ) or getattr(nb_session, "notebook_session_configuration_details", None)
 
                 if nb_config:
                     self._load_infra_from_notebook(nb_config)
@@ -294,9 +305,9 @@ class DSCJob(OCIDataScienceMixin, oci.data_science.models.Job):
         return self
 
     def _create_with_oci_api(self) -> None:
-        res = self.client.create_job(
-            self.to_oci_model(oci.data_science.models.CreateJobDetails)
-        )
+        oci_model = self.to_oci_model(oci.data_science.models.CreateJobDetails)
+        logger.debug(oci_model)
+        res = self.client.create_job(oci_model)
         self.update_from_oci_model(res.data)
         if self.lifecycle_state == "ACTIVE":
             return
@@ -345,9 +356,16 @@ class DSCJob(OCIDataScienceMixin, oci.data_science.models.Job):
         """Updates the Data Science Job."""
         raise NotImplementedError("Updating Job is not supported at the moment.")
 
-    def delete(self) -> DSCJob:
+    def delete(self, force_delete: bool = False) -> DSCJob:
         """Deletes the job and the corresponding job runs.
 
+        Parameters
+        ----------
+        force_delete : bool, optional, defaults to False
+            the deletion fails when associated job runs are in progress, but if force_delete to true, then
+            the job run will be canceled, then it will be deleted. In this case, delete job has to wait till
+            job has been canceled.
+
         Returns
         -------
         DSCJob
@@ -356,6 +374,12 @@ class DSCJob(OCIDataScienceMixin, oci.data_science.models.Job):
         """
         runs = self.run_list()
         for run in runs:
+            if run.lifecycle_state in [
+                DataScienceJobRun.LIFECYCLE_STATE_ACCEPTED,
+                DataScienceJobRun.LIFECYCLE_STATE_IN_PROGRESS,
+                DataScienceJobRun.LIFECYCLE_STATE_NEEDS_ATTENTION,
+            ]:
+                run.cancel(wait_for_completion=True)
             run.delete()
         self.client.delete_job(self.id)
         return self
@@ -422,7 +446,7 @@ class DSCJob(OCIDataScienceMixin, oci.data_science.models.Job):
         items = oci.pagination.list_call_get_all_results(
             self.client.list_job_runs, self.compartment_id, job_id=self.id, **kwargs
         ).data
-        return [DataScienceJobRun.from_oci_model(item) for item in items]
+        return [DataScienceJobRun(**self.auth).from_oci_model(item) for item in items]
 
     def run(self, **kwargs) -> DataScienceJobRun:
         """Runs the job
@@ -437,6 +461,8 @@ class DSCJob(OCIDataScienceMixin, oci.data_science.models.Job):
             * command_line_arguments: str
             * maximum_runtime_in_minutes: int
             * display_name: str
+            * freeform_tags: dict(str, str)
+            * defined_tags: dict(str, dict(str, object))
 
         If display_name is not specified, it will be generated as "<JOB_NAME>-run-<TIMESTAMP>".
 
@@ -486,28 +512,11 @@ class DSCJob(OCIDataScienceMixin, oci.data_science.models.Job):
             kwargs["job_configuration_override_details"] = config_override
 
         wait = kwargs.pop("wait", False)
-        run = DataScienceJobRun(**kwargs).create()
+        run = DataScienceJobRun(**kwargs, **self.auth).create()
         if wait:
             return run.watch()
         return run
 
-    @classmethod
-    def from_ocid(cls, ocid) -> DSCJob:
-        """Gets a job by OCID
-
-        Parameters
-        ----------
-        ocid : str
-            The OCID of the job.
-
-        Returns
-        -------
-        DSCJob
-            An instance of DSCJob.
-
-        """
-        return super().from_ocid(ocid)
-
 
 class DataScienceJobRun(
     OCIDataScienceMixin, oci.data_science.models.JobRun, RunInstance
@@ -565,7 +574,12 @@ class DataScienceJobRun(
         if not self.log_id:
             raise ValueError("Log OCID is not specified for this job run.")
         # Specifying log group ID when initializing OCILog can reduce the number of API calls.
-        return OCILog(id=self.log_id, log_group_id=self.log_details.log_group_id)
+        auth = self.auth
+        if "client_kwargs" in auth and isinstance(auth["client_kwargs"], dict):
+            auth["client_kwargs"].pop("service_endpoint", None)
+        return OCILog(
+            id=self.log_id, log_group_id=self.log_details.log_group_id, **auth
+        )
 
     @staticmethod
     def _format_log(message: str, date_time: datetime.datetime) -> dict:
@@ -714,9 +728,14 @@ class DataScienceJobRun(
 
         return self
 
-    def cancel(self) -> DataScienceJobRun:
+    def cancel(self, wait_for_completion: bool = True) -> DataScienceJobRun:
         """Cancels a job run
-        This method will wait for the job run to be canceled before returning.
+
+        Parameters
+        ----------
+        wait_for_completion: bool
+            Whether to wait for job run to be cancelled before proceeding.
+            Defaults to True.
 
         Returns
         -------
@@ -724,9 +743,13 @@ class DataScienceJobRun(
             The job run instance.
         """
         self.client.cancel_job_run(self.id)
-        while self.lifecycle_state != "CANCELED":
-            self.sync()
-            time.sleep(SLEEP_INTERVAL)
+        if wait_for_completion:
+            while (
+                self.lifecycle_state
+                != oci.data_science.models.JobRun.LIFECYCLE_STATE_CANCELED
+            ):
+                self.sync()
+                time.sleep(SLEEP_INTERVAL)
         return self
 
     def __repr__(self) -> str:
@@ -833,6 +856,16 @@ class DataScienceJob(Infrastructure):
             .with_shape_config_details(memory_in_gbs=16, ocpus=1)
             # Minimum/Default block storage size is 50 (GB).
             .with_block_storage_size(50)
+            # A list of file systems to be mounted
+            .with_storage_mount(
+                {
+                    "src" : "<mount_target_ip_address>:<export_path>",
+                    "dest" : "<destination_directory_name>"
+                }
+            )
+            # Tags
+            .with_freeform_tag(my_tag="my_value")
+            .with_defined_tag(**{"Operations": {"CostCenter": "42"}})
         )
 
     """
@@ -850,6 +883,9 @@ class DataScienceJob(Infrastructure):
     CONST_OCPUS = "ocpus"
     CONST_LOG_ID = "logId"
     CONST_LOG_GROUP_ID = "logGroupId"
+    CONST_STORAGE_MOUNT = "storageMount"
+    CONST_FREEFORM_TAGS = "freeformTags"
+    CONST_DEFINED_TAGS = "definedTags"
 
     attribute_map = {
         CONST_PROJECT_ID: "project_id",
@@ -863,6 +899,9 @@ class DataScienceJob(Infrastructure):
         CONST_SHAPE_CONFIG_DETAILS: "shape_config_details",
         CONST_LOG_ID: "log_id",
         CONST_LOG_GROUP_ID: "log_group_id",
+        CONST_STORAGE_MOUNT: "storage_mount",
+        CONST_FREEFORM_TAGS: "freeform_tags",
+        CONST_DEFINED_TAGS: "defined_tags",
     }
 
     shape_config_details_attribute_map = {
@@ -888,6 +927,13 @@ class DataScienceJob(Infrastructure):
         v.split(".", maxsplit=1)[-1]: k for k, v in payload_attribute_map.items()
     }
 
+    storage_mount_type_dict = {
+        FILE_STORAGE_TYPE: OCIFileStorage,
+        OBJECT_STORAGE_TYPE: OCIObjectStorage,
+    }
+
+    auth = {}
+
     @staticmethod
     def standardize_spec(spec):
         if not spec:
@@ -898,16 +944,20 @@ class DataScienceJob(Infrastructure):
             **DataScienceJob.shape_config_details_attribute_map,
         }
         snake_to_camel_map = {v: k for k, v in attribute_map.items()}
+        snake_to_camel_map = {
+            **{v: k for k, v in attribute_map.items()},
+            **DataScienceJob.snake_to_camel_map,
+        }
 
         for key in list(spec.keys()):
-            if key not in attribute_map and key in snake_to_camel_map:
+            if key not in attribute_map and key.lower() in snake_to_camel_map:
                 value = spec.pop(key)
                 if isinstance(value, dict):
-                    spec[snake_to_camel_map[key]] = DataScienceJob.standardize_spec(
-                        value
-                    )
+                    spec[
+                        snake_to_camel_map[key.lower()]
+                    ] = DataScienceJob.standardize_spec(value)
                 else:
-                    spec[snake_to_camel_map[key]] = value
+                    spec[snake_to_camel_map[key.lower()]] = value
         return spec
 
     def __init__(self, spec: Dict = None, **kwargs) -> None:
@@ -921,12 +971,16 @@ class DataScienceJob(Infrastructure):
             Specification as keyword arguments.
             If spec contains the same key as the one in kwargs, the value from kwargs will be used.
         """
+        for key in ["config", "signer", "client_kwargs"]:
+            if kwargs.get(key):
+                self.auth[key] = kwargs.pop(key)
+
         self.standardize_spec(spec)
         self.standardize_spec(kwargs)
         super().__init__(spec=spec, **kwargs)
         if not self.job_type:
             self.with_job_type("DEFAULT")
-        self.dsc_job = DSCJob()
+        self.dsc_job = DSCJob(**self.auth)
         self.runtime = None
         self._name = None
 
@@ -1212,6 +1266,74 @@ class DataScienceJob(Infrastructure):
         """
         return self.get_spec(self.CONST_LOG_GROUP_ID)
 
+    def with_storage_mount(self, *storage_mount: List[dict]) -> DataScienceJob:
+        """Sets the file systems to be mounted for the data science job.
+        A maximum number of 5 file systems are allowed to be mounted for a single data science job.
+
+        Parameters
+        ----------
+        storage_mount : List[dict]
+            A list of file systems to be mounted.
+
+        Returns
+        -------
+        DataScienceJob
+            The DataScienceJob instance (self)
+        """
+        storage_mount_list = []
+        for item in storage_mount:
+            if not isinstance(item, dict):
+                raise ValueError(
+                    "Parameter `storage_mount` should be a list of dictionaries."
+                )
+            storage_mount_list.append(item)
+        if len(storage_mount_list) > MAXIMUM_MOUNT_COUNT:
+            raise ValueError(
+                f"A maximum number of {MAXIMUM_MOUNT_COUNT} file systems are allowed to be mounted at this time for a job."
+            )
+        return self.set_spec(self.CONST_STORAGE_MOUNT, storage_mount_list)
+
+    @property
+    def storage_mount(self) -> List[dict]:
+        """Files systems that have been mounted for the data science job
+
+        Returns
+        -------
+        list
+            A list of file systems that have been mounted
+        """
+        return self.get_spec(self.CONST_STORAGE_MOUNT, [])
+
+    def with_freeform_tag(self, **kwargs) -> DataScienceJob:
+        """Sets freeform tags
+
+        Returns
+        -------
+        DataScienceJob
+            The DataScienceJob instance (self)
+        """
+        return self.set_spec(self.CONST_FREEFORM_TAGS, kwargs)
+
+    def with_defined_tag(self, **kwargs) -> DataScienceJob:
+        """Sets defined tags
+
+        Returns
+        -------
+        DataScienceJob
+            The DataScienceJob instance (self)
+        """
+        return self.set_spec(self.CONST_DEFINED_TAGS, kwargs)
+
+    @property
+    def freeform_tags(self) -> dict:
+        """Freeform tags"""
+        return self.get_spec(self.CONST_FREEFORM_TAGS, {})
+
+    @property
+    def defined_tags(self) -> dict:
+        """Defined tags"""
+        return self.get_spec(self.CONST_DEFINED_TAGS, {})
+
     def _prepare_log_config(self) -> dict:
         if not self.log_group_id and not self.log_id:
             return None
@@ -1285,6 +1407,42 @@ class DataScienceJob(Infrastructure):
                         sub_spec[sub_infra_attr] = sub_value
                 if sub_spec:
                     self._spec[infra_attr] = sub_spec
+
+        self._update_storage_mount_from_dsc_model(dsc_job, overwrite)
+        return self
+
+    def _update_storage_mount_from_dsc_model(
+        self, dsc_job: oci.data_science.models.Job, overwrite: bool = True
+    ) -> DataScienceJob:
+        """Update the mount storage properties from an OCI data science job model.
+
+        Parameters
+        ----------
+        dsc_job: oci.data_science.models.Job
+            An OCI data science job model.
+
+        overwrite: bool
+            Whether to overwrite the existing values.
+            If this is set to False, only the empty/None properties will be updated.
+
+        Returns
+        -------
+        DataScienceJob
+            The DataScienceJob instance (self)
+        """
+        storage_mount_list = get_value(
+            dsc_job, "job_storage_mount_configuration_details_list"
+        )
+        if storage_mount_list:
+            storage_mount = [
+                self.storage_mount_type_dict[
+                    file_system.storage_type
+                ].update_from_dsc_model(file_system)
+                for file_system in storage_mount_list
+                if file_system.storage_type in self.storage_mount_type_dict
+            ]
+            if overwrite or not self.get_spec(self.CONST_STORAGE_MOUNT):
+                self.set_spec(self.CONST_STORAGE_MOUNT, storage_mount)
         return self
 
     def _update_job_infra(self, dsc_job: DSCJob) -> DataScienceJob:
@@ -1317,17 +1475,60 @@ class DataScienceJob(Infrastructure):
             if value:
                 dsc_job.job_infrastructure_configuration_details[camel_attr] = value
 
+        if not dsc_job.job_infrastructure_configuration_details.get(
+            "shapeName", ""
+        ).endswith("Flex") and dsc_job.job_infrastructure_configuration_details.get(
+            "jobShapeConfigDetails"
+        ):
+            raise ValueError(
+                "Shape config is not required for non flex shape from user end."
+            )
+
         if dsc_job.job_infrastructure_configuration_details.get("subnetId"):
             dsc_job.job_infrastructure_configuration_details[
                 "jobInfrastructureType"
             ] = JobInfrastructureConfigurationDetails.JOB_INFRASTRUCTURE_TYPE_STANDALONE
+
+        if self.storage_mount:
+            if not hasattr(oci.data_science.models, "StorageMountConfigurationDetails"):
+                raise EnvironmentError(
+                    "Storage mount hasn't been supported in the current OCI SDK installed."
+                )
+            dsc_job.job_storage_mount_configuration_details_list = [
+                DSCFileSystemManager.initialize(file_system)
+                for file_system in self.storage_mount
+            ]
         return self
 
     def build(self) -> DataScienceJob:
         self.dsc_job.load_defaults()
+
+        try:
+            self.dsc_job.load_defaults()
+        except Exception:
+            logger.exception("Failed to load default properties.")
+
         self._update_from_dsc_model(self.dsc_job, overwrite=False)
         return self
 
+    def init(self, **kwargs) -> DataScienceJob:
+        """Initializes a starter specification for the DataScienceJob.
+
+        Returns
+        -------
+        DataScienceJob
+            The DataScienceJob instance (self)
+        """
+        return (
+            self.build()
+            .with_compartment_id(self.compartment_id or "{Provide a compartment OCID}")
+            .with_project_id(self.project_id or "{Provide a project OCID}")
+            .with_subnet_id(
+                self.subnet_id
+                or "{Provide a subnet OCID or remove this field if you use a default networking}"
+            )
+        )
+
     def create(self, runtime, **kwargs) -> DataScienceJob:
         """Creates a job with runtime.
 
@@ -1361,8 +1562,12 @@ class DataScienceJob(Infrastructure):
 
         payload["display_name"] = display_name
         payload["job_log_configuration_details"] = self._prepare_log_config()
+        if not payload.get("freeform_tags"):
+            payload["freeform_tags"] = self.freeform_tags
+        if not payload.get("defined_tags"):
+            payload["defined_tags"] = self.defined_tags
 
-        self.dsc_job = DSCJob(**payload)
+        self.dsc_job = DSCJob(**payload, **self.auth)
         # Set Job infra to user values after DSCJob initialized the defaults
         self._update_job_infra(self.dsc_job)
         self.dsc_job.create()
@@ -1371,7 +1576,14 @@ class DataScienceJob(Infrastructure):
         return self
 
     def run(
-        self, name=None, args=None, env_var=None, freeform_tags=None, wait=False
+        self,
+        name=None,
+        args=None,
+        env_var=None,
+        freeform_tags=None,
+        defined_tags=None,
+        wait=False,
+        **kwargs,
     ) -> DataScienceJobRun:
         """Runs a job on OCI Data Science job
 
@@ -1385,8 +1597,12 @@ class DataScienceJob(Infrastructure):
             Environment variable for the job run, by default None
         freeform_tags : dict, optional
             Freeform tags for the job run, by default None
+        defined_tags : dict, optional
+            Defined tags for the job run, by default None
         wait : bool, optional
             Indicate if this method should wait for the run to finish before it returns, by default False.
+        kwargs
+            additional keyword arguments
 
         Returns
         -------
@@ -1399,11 +1615,18 @@ class DataScienceJob(Infrastructure):
             raise RuntimeError(
                 "Job is not created. Call create() to create the job first."
             )
-        tags = self.runtime.freeform_tags
-        if not tags:
-            tags = {}
-        if freeform_tags:
-            tags.update(freeform_tags)
+
+        if not freeform_tags:
+            freeform_tags = {}
+        runtime_freeform_tags = self.runtime.freeform_tags
+        if runtime_freeform_tags:
+            freeform_tags.update(runtime_freeform_tags)
+
+        if not defined_tags:
+            defined_tags = {}
+        runtime_defined_tags = self.runtime.defined_tags
+        if runtime_defined_tags:
+            defined_tags.update(runtime_defined_tags)
 
         if name:
             envs = self.runtime.envs
@@ -1411,13 +1634,21 @@ class DataScienceJob(Infrastructure):
                 envs.update(env_var)
             name = Template(name).safe_substitute(envs)
 
-        return self.dsc_job.run(
+        kwargs = dict(
             display_name=name,
             command_line_arguments=args,
             environment_variables=env_var,
-            freeform_tags=tags,
+            freeform_tags=freeform_tags,
+            defined_tags=defined_tags,
             wait=wait,
+            **kwargs,
         )
+        # A Runtime class may define customized run() method.
+        # Use the customized method if the run() method is defined by the runtime.
+        # Otherwise, use the default run() method defined in this class.
+        if hasattr(self.runtime, "run"):
+            return self.runtime.run(self.dsc_job, **kwargs)
+        return self.dsc_job.run(**kwargs)
 
     def delete(self) -> None:
         """Deletes a job"""
@@ -1461,7 +1692,7 @@ class DataScienceJob(Infrastructure):
         instance.runtime = DataScienceJobRuntimeManager(instance).extract(dsc_job)
         return instance
 
-    @classmethod
+    @class_or_instance_method
     def from_id(cls, job_id: str) -> DataScienceJob:
         """Gets an existing job using Job OCID
 
@@ -1477,9 +1708,9 @@ class DataScienceJob(Infrastructure):
             An instance of DataScienceJob
 
         """
-        return cls.from_dsc_job(DSCJob.from_ocid(job_id))
+        return cls.from_dsc_job(DSCJob(**cls.auth).from_ocid(job_id))
 
-    @classmethod
+    @class_or_instance_method
     def list_jobs(cls, compartment_id: str = None, **kwargs) -> List[DataScienceJob]:
         """Lists all jobs in a compartment.
 
@@ -1500,10 +1731,10 @@ class DataScienceJob(Infrastructure):
         """
         return [
             cls.from_dsc_job(job)
-            for job in DSCJob.list_resource(compartment_id, **kwargs)
+            for job in DSCJob(**cls.auth).list_resource(compartment_id, **kwargs)
         ]
 
-    @classmethod
+    @class_or_instance_method
     def instance_shapes(cls, compartment_id: str = None, **kwargs) -> list:
         """Lists the supported shapes for running jobs in a compartment.
 
@@ -1531,13 +1762,13 @@ class DataScienceJob(Infrastructure):
 
         """
         shapes = oci.pagination.list_call_get_all_results(
-            DSCJob.init_client().list_job_shapes,
+            DSCJob(**cls.auth).init_client().list_job_shapes,
             DSCJob.check_compartment_id(compartment_id),
             **kwargs,
         ).data
         return shapes
 
-    @classmethod
+    @class_or_instance_method
     def fast_launch_shapes(cls, compartment_id: str = None, **kwargs) -> list:
         """Lists the supported fast launch shapes for running jobs in a compartment.
 
@@ -1565,7 +1796,7 @@ class DataScienceJob(Infrastructure):
 
         """
         shapes = oci.pagination.list_call_get_all_results(
-            DSCJob.init_client().list_fast_launch_job_configs,
+            DSCJob(**cls.auth).init_client().list_fast_launch_job_configs,
             DSCJob.check_compartment_id(compartment_id),
             **kwargs,
         ).data
