@@ -11,9 +11,11 @@ import sys
 import shutil
 import tempfile
 import uuid
+import fsspec
 from typing import Dict, Optional, Tuple
 from ads.common import auth as authutil
 from ads.common import logger, utils
+from ads.common.object_storage_details import ObjectStorageDetails
 from ads.config import CONDA_BUCKET_NAME, CONDA_BUCKET_NS
 from ads.model.runtime.env_info import EnvInfo, InferenceEnvInfo, TrainingEnvInfo
 from ads.model.runtime.runtime_info import RuntimeInfo
@@ -28,13 +30,13 @@ SCORE_VERSION = "1.0"
 ADS_VERSION = __version__
 
 
-class ArtifactNestedFolderError(Exception):
+class ArtifactNestedFolderError(Exception):  # pragma: no cover
     def __init__(self, folder: str):
         self.folder = folder
         super().__init__("The required artifact files placed in a nested folder.")
 
 
-class ArtifactRequiredFilesError(Exception):
+class ArtifactRequiredFilesError(Exception):  # pragma: no cover
     def __init__(self, required_files: Tuple[str]):
         super().__init__(
             "Not all required files presented in artifact folder. "
@@ -42,7 +44,7 @@ class ArtifactRequiredFilesError(Exception):
         )
 
 
-class AritfactFolderStructureError(Exception):
+class AritfactFolderStructureError(Exception):  # pragma: no cover
     def __init__(self, required_files: Tuple[str]):
         super().__init__(
             "The artifact folder has a wrong structure. "
@@ -116,17 +118,27 @@ class ModelArtifact:
         model_file_name: str = None,
         reload: Optional[bool] = False,
         ignore_conda_error: Optional[bool] = False,
+        local_copy_dir: str = None,
+        auth: dict = None,
     ):
         """Initializes a ModelArtifact instance.
 
         Parameters
         ----------
         artifact_dir: str
-            The local artifact folder to store the files needed for deployment.
+            The artifact folder to store the files needed for deployment.
         model_file_name: (str, optional). Defaults to `None`.
             The file name of the serialized model.
         reload: (bool, optional). Defaults to False.
             Determine whether will reload the Model into the env.
+        ignore_conda_error: (bool, optional). Defaults to False.
+            Parameter to ignore error when collecting conda information.
+        local_copy_dir: (str, optional). Defaults to None.
+            The local back up directory of the model artifacts.
+        auth :(Dict, optional). Defaults to None.
+            The default authetication is set using `ads.set_auth` API. If you need to override the
+            default, use the `ads.common.auth.api_keys` or `ads.common.auth.resource_principal` to create appropriate
+            authentication signer and kwargs required to instantiate IdentityClient object.
 
         Returns
         -------
@@ -141,13 +153,25 @@ class ModelArtifact:
         if not artifact_dir:
             raise ValueError("The `artifact_dir` needs to be provided.")
 
+        self.artifact_dir = (
+            artifact_dir
+            if ObjectStorageDetails.is_oci_path(artifact_dir)
+            else os.path.abspath(os.path.expanduser(artifact_dir))
+        )
+        self.local_copy_dir = (
+            local_copy_dir or tempfile.mkdtemp()
+            if ObjectStorageDetails.is_oci_path(artifact_dir)
+            else artifact_dir
+        )
+
         self.score = None
-        self.artifact_dir = os.path.abspath(os.path.expanduser(artifact_dir))
         sys.path.insert(0, self.artifact_dir)
         self.model_file_name = model_file_name
         self._env = Environment(loader=PackageLoader("ads", "templates"))
         self.ignore_conda_error = ignore_conda_error
         self.model = None
+        self.auth = auth or authutil.default_signer()
+
         if reload and not ignore_conda_error:
             self.reload()
             # Extracts the model_file_name from the score.py.
@@ -215,7 +239,7 @@ class ModelArtifact:
             runtime_info.model_provenance.training_code.artifact_directory = (
                 self.artifact_dir
             )
-            runtime_info.save()
+            runtime_info.save(storage_options=auth)
             return runtime_info
         inference_conda_env = ModelArtifact._populate_env_info(
             InferenceEnvInfo,
@@ -249,8 +273,9 @@ class ModelArtifact:
             or runtime_info.model_deployment.inference_conda_env.inference_python_version.strip()
             == ""
         ):
-            warnings.warn(
-                "Cannot automatically detect the inference python version. `inference_python_version` must be provided."
+            raise ValueError(
+                "Cannot automatically detect the inference python version. "
+                "`inference_python_version` must be provided."
             )
         runtime_file_path = os.path.join(self.artifact_dir, "runtime.yaml")
         if os.path.exists(runtime_file_path) and not force_overwrite:
@@ -259,7 +284,7 @@ class ModelArtifact:
                 "Set `force_overwrite` to True to overwrite all the files."
             )
         else:
-            runtime_info.save()
+            runtime_info.save(storage_options=auth)
         return runtime_info
 
     @staticmethod
@@ -345,9 +370,13 @@ class ModelArtifact:
             "ADS_VERSION": ADS_VERSION,
             "time_created": time_suffix,
         }
+        storage_options = kwargs.pop("auth", {})
+        storage_options = storage_options if storage_options else {}
         context.update(kwargs)
-        with open(os.path.join(self.artifact_dir, "score.py"), "w") as sfl:
-            sfl.write(scorefn_template.render(context))
+        with fsspec.open(
+            os.path.join(self.artifact_dir, "score.py"), "w", **storage_options
+        ) as f:
+            f.write(scorefn_template.render(context))
 
     def reload(self):
         """Syncs the `score.py` to reload the model and predict function.
@@ -356,17 +385,26 @@ class ModelArtifact:
         -------
         None
             Nothing
+
         """
+        if ObjectStorageDetails.is_oci_path(self.artifact_dir):
+            utils.copy_from_uri(
+                uri=self.artifact_dir,
+                to_path=self.local_copy_dir,
+                force_overwrite=True,
+                auth=self.auth,
+            )
+
         spec = importlib.util.spec_from_file_location(
-            "score%s" % uuid.uuid4(), os.path.join(self.artifact_dir, "score.py")
+            "score%s" % uuid.uuid4(), os.path.join(self.local_copy_dir, "score.py")
         )
         self.score = importlib.util.module_from_spec(spec)
         spec.loader.exec_module(self.score)
         self.model = self.score.load_model()  # load model in cache
         # remove the cache files.
         for dir in [
-            os.path.join(self.artifact_dir, "__pycache__"),
-            os.path.join(self.artifact_dir, ".ipynb_checkpoints"),
+            os.path.join(self.local_copy_dir, "__pycache__"),
+            os.path.join(self.local_copy_dir, ".ipynb_checkpoints"),
         ]:
             if os.path.exists(dir):
                 shutil.rmtree(dir, ignore_errors=True)
@@ -380,6 +418,7 @@ class ModelArtifact:
         force_overwrite: Optional[bool] = False,
         auth: Optional[Dict] = None,
         ignore_conda_error: Optional[bool] = False,
+        reload: Optional[bool] = False,
     ):
         """Constructs a ModelArtifact object from the existing model artifacts.
 
@@ -390,16 +429,20 @@ class ModelArtifact:
             OCI object storage URI.
         artifact_dir: str
             The local artifact folder to store the files needed for deployment.
-        model_file_name: (str, optional). Defaults to `None`
-            The file name of the serialized model.
-        force_overwrite: (bool, optional). Defaults to False.
-            Whether to overwrite existing files or not.
         auth: (Dict, optional). Defaults to None.
             The default authetication is set using `ads.set_auth` API.
             If you need to override the default, use the `ads.common.auth.api_keys`
             or `ads.common.auth.resource_principal` to create appropriate
             authentication signer and kwargs required to instantiate
             IdentityClient object.
+        force_overwrite: (bool, optional). Defaults to False.
+            Whether to overwrite existing files or not.
+        ignore_conda_error: (bool, optional). Defaults to False.
+            Parameter to ignore error when collecting conda information.
+        model_file_name: (str, optional). Defaults to `None`
+            The file name of the serialized model.
+        reload: (bool, optional). Defaults to False.
+            Whether to reload the Model into the environment.
 
         Returns
         -------
@@ -410,42 +453,73 @@ class ModelArtifact:
         ------
         ValueError
             If `uri` is equal to `artifact_dir`, and it not exists.
+            If `artifact_dir` is not provided.
         """
-        artifact_dir = os.path.join(
-            os.path.abspath(os.path.expanduser(artifact_dir)), ""
+        if not artifact_dir:
+            raise ValueError("The `artifact_dir` needs to be provided.")
+
+        artifact_dir = (
+            artifact_dir
+            if ObjectStorageDetails.is_oci_path(artifact_dir)
+            else os.path.join(os.path.abspath(os.path.expanduser(artifact_dir)), "")
         )
 
-        if artifact_dir == os.path.join(
-            os.path.abspath(os.path.expanduser(uri)).rstrip("/"), ""
-        ):
-            if not os.path.exists(artifact_dir):
+        if not ObjectStorageDetails.is_oci_path(uri):
+            uri = os.path.join(os.path.abspath(os.path.expanduser(uri)).rstrip("/"), "")
+        auth = auth or authutil.default_signer()
+
+        to_path = (
+            tempfile.mkdtemp()
+            if ObjectStorageDetails.is_oci_path(artifact_dir)
+            else artifact_dir
+        )
+        force_overwrite = (
+            True if ObjectStorageDetails.is_oci_path(artifact_dir) else force_overwrite
+        )
+        if artifact_dir == uri and not ObjectStorageDetails.is_oci_path(artifact_dir):
+            if not utils.is_path_exists(artifact_dir, auth=auth):
                 raise ValueError("Provided `uri` doesn't exist.")
         else:
-            auth = auth or authutil.default_signer()
             utils.copy_from_uri(
                 uri=uri,
-                to_path=artifact_dir,
+                to_path=to_path,
                 unpack=True,
                 force_overwrite=force_overwrite,
                 auth=auth,
             )
+
         if not ignore_conda_error:
             try:
-                _validate_artifact_dir(artifact_dir)
+                _validate_artifact_dir(to_path)
             except ArtifactNestedFolderError as exc:
                 with tempfile.TemporaryDirectory() as temp_dir:
                     utils.copy_from_uri(
                         uri=exc.folder, to_path=temp_dir, force_overwrite=True
                     )
                     utils.copy_from_uri(
-                        uri=temp_dir, to_path=artifact_dir, force_overwrite=True
+                        uri=temp_dir, to_path=to_path, force_overwrite=True
+                    )
+            except ArtifactRequiredFilesError as ex:
+                logger.warning(ex)
+
+        if ObjectStorageDetails.is_oci_path(artifact_dir):
+            for root, dirs, files in os.walk(to_path):
+                prefix = (os.path.abspath(root).split(to_path)[-1]).lstrip("/")
+                for file in files:
+                    path = os.path.join(prefix, file)
+                    utils.copy_file(
+                        uri_src=os.path.join(root, file),
+                        uri_dst=os.path.join(artifact_dir, path),
+                        force_overwrite=True,
+                        auth=auth,
                     )
 
         return cls(
-            artifact_dir,
-            model_file_name,
-            reload=True,
+            artifact_dir=artifact_dir,
             ignore_conda_error=ignore_conda_error,
+            local_copy_dir=to_path,
+            model_file_name=model_file_name,
+            reload=reload,
         )
 
     def __getattr__(self, item):
