@@ -1,12 +1,11 @@
 #!/usr/bin/env python
 # -*- coding: utf-8; -*-
 
-# Copyright (c) 2022, 2023 Oracle and/or its affiliates.
+# Copyright (c) 2022, 2024 Oracle and/or its affiliates.
 # Licensed under the Universal Permissive License v 1.0 as shown at https://oss.oracle.com/licenses/upl/
 
 import os
 import shutil
-import tempfile
 import uuid
 from abc import ABC, abstractmethod
 from typing import Dict, Optional
@@ -15,6 +14,7 @@ from zipfile import ZipFile
 from ads.common import utils
 from ads.common.utils import extract_region
 from ads.model.service.oci_datascience_model import OCIDataScienceModel
+from ads.common.object_storage_details import ObjectStorageDetails
 
 
 class ArtifactDownloader(ABC):
@@ -56,13 +56,14 @@ class ArtifactDownloader(ABC):
         ValueError
             If target directory does not exist.
         """
-        if os.path.exists(self.target_dir):
+        if os.path.exists(self.target_dir) and len(os.listdir(self.target_dir)) > 0:
             if not self.force_overwrite:
                 raise ValueError(
                     f"The `{self.target_dir}` directory already exists. "
                     "Set `force_overwrite` to `True` if you wish to overwrite."
                 )
             shutil.rmtree(self.target_dir)
+        os.makedirs(self.target_dir, exist_ok=True)
         with utils.get_progress_bar(
             ArtifactDownloader.PROGRESS_STEPS_COUNT + self.PROGRESS_STEPS_COUNT
         ) as progress:
@@ -84,15 +85,31 @@ class SmallArtifactDownloader(ArtifactDownloader):
     def _download(self):
         """Downloads model artifacts."""
         self.progress.update("Importing model artifacts from catalog")
-        zip_content = self.dsc_model.get_model_artifact_content()
-        with tempfile.TemporaryDirectory() as temp_dir:
-            self.progress.update("Copying model artifacts to the artifact directory")
-            zip_file_path = os.path.join(temp_dir, f"{str(uuid.uuid4())}.zip")
-            with open(zip_file_path, "wb") as zip_file:
-                zip_file.write(zip_content)
+
+        artifact_info = self.dsc_model.get_artifact_info()
+        artifact_name = artifact_info["Content-Disposition"].replace(
+            "attachment; filename=", ""
+        )
+        _, file_extension = os.path.splitext(artifact_name)
+        file_extension = file_extension.lower() if file_extension else ".zip"
+
+        file_content = self.dsc_model.get_model_artifact_content()
+        self.progress.update("Copying model artifacts to the artifact directory")
+
+        file_name = (
+            "model_description" if file_extension == ".json" else str(uuid.uuid4())
+        )
+        artifact_file_path = os.path.join(
+            self.target_dir, f"{file_name}{file_extension}"
+        )
+        with open(artifact_file_path, "wb") as _file:
+            _file.write(file_content)
+
+        if file_extension == ".zip":
             self.progress.update("Extracting model artifacts")
-            with ZipFile(zip_file_path) as zip_file:
-                zip_file.extractall(self.target_dir)
+            with ZipFile(artifact_file_path) as _file:
+                _file.extractall(self.target_dir)
+            utils.remove_file(artifact_file_path)
 
 
 class LargeArtifactDownloader(ArtifactDownloader):
@@ -108,6 +125,7 @@ class LargeArtifactDownloader(ArtifactDownloader):
         bucket_uri: Optional[str] = None,
         overwrite_existing_artifact: Optional[bool] = True,
         remove_existing_artifact: Optional[bool] = True,
+        model_file_description: Optional[dict] = None,
     ):
         """Initializes `LargeArtifactDownloader` instance.
 
@@ -135,6 +153,9 @@ class LargeArtifactDownloader(ArtifactDownloader):
             Overwrite target bucket artifact if exists.
         remove_existing_artifact: (bool, optional). Defaults to `True`.
             Wether artifacts uploaded to object storage bucket need to be removed or not.
+        model_file_description: (dict, optional). Defaults to None.
+            Contains object path details for models created by reference.
+
         """
         super().__init__(
             dsc_model=dsc_model, target_dir=target_dir, force_overwrite=force_overwrite
@@ -144,12 +165,19 @@ class LargeArtifactDownloader(ArtifactDownloader):
         self.bucket_uri = bucket_uri
         self.overwrite_existing_artifact = overwrite_existing_artifact
         self.remove_existing_artifact = remove_existing_artifact
+        self.model_file_description = model_file_description
 
     def _download(self):
         """Downloads model artifacts."""
         self.progress.update(f"Importing model artifacts from catalog")
 
+        if self.dsc_model.is_model_by_reference() and self.model_file_description:
+            self.download_from_model_file_description()
+            self.progress.update()
+            return
+
         bucket_uri = self.bucket_uri
+
         if not os.path.basename(bucket_uri):
             bucket_uri = os.path.join(bucket_uri, f"{self.dsc_model.id}.zip")
         elif not bucket_uri.lower().endswith(".zip"):
@@ -157,18 +185,18 @@ class LargeArtifactDownloader(ArtifactDownloader):
 
         self.dsc_model.import_model_artifact(bucket_uri=bucket_uri, region=self.region)
         self.progress.update("Copying model artifacts to the artifact directory")
-        with tempfile.TemporaryDirectory() as temp_dir:
-            zip_file_path = os.path.join(temp_dir, f"{str(uuid.uuid4())}.zip")
-            zip_file_path = utils.copy_file(
-                uri_src=bucket_uri,
-                uri_dst=zip_file_path,
-                auth=self.auth,
-                progressbar_description="Copying model artifacts to the artifact directory",
-            )
-            self.progress.update("Extracting model artifacts")
-            with ZipFile(zip_file_path) as zip_file:
-                zip_file.extractall(self.target_dir)
+        zip_file_path = os.path.join(self.target_dir, f"{str(uuid.uuid4())}.zip")
+        zip_file_path = utils.copy_file(
+            uri_src=bucket_uri,
+            uri_dst=zip_file_path,
+            auth=self.auth,
+            progressbar_description="Copying model artifacts to the artifact directory",
+        )
+        self.progress.update("Extracting model artifacts")
+        with ZipFile(zip_file_path) as zip_file:
+            zip_file.extractall(self.target_dir)
 
+        utils.remove_file(zip_file_path)
         if self.remove_existing_artifact:
             self.progress.update(
                 "Removing temporary artifacts from the Object Storage bucket"
@@ -176,3 +204,49 @@ class LargeArtifactDownloader(ArtifactDownloader):
             utils.remove_file(bucket_uri)
         else:
             self.progress.update()
+
+    def download_from_model_file_description(self):
+        """Helper function to download the objects using model file description content to the target directory."""
+
+        models = self.model_file_description["models"]
+        total_size = 0
+
+        for model in models:
+            namespace, bucket_name, prefix = (
+                model["namespace"],
+                model["bucketName"],
+                model["prefix"],
+            )
+            bucket_uri = f"oci://{bucket_name}@{namespace}/{prefix}"
+
+            message = f"Copying model artifacts by reference from {bucket_uri} to {self.target_dir}"
+            self.progress.update(message)
+
+            objects = model["objects"]
+            os_details_list = list()
+
+            for obj in objects:
+                name = obj["name"]
+                version = None if obj["version"] == "" else obj["version"]
+                size = obj["sizeInBytes"]
+                if size == 0:
+                    continue
+                total_size += size
+                object_uri = f"oci://{bucket_name}@{namespace}/{name}"
+
+                os_details = ObjectStorageDetails.from_path(object_uri)
+                os_details.version = version
+                os_details_list.append(os_details)
+            try:
+                ObjectStorageDetails.from_path(
+                    bucket_uri
+                ).bulk_download_from_object_storage(
+                    paths=os_details_list,
+                    target_dir=self.target_dir,
+                    progress_bar=self.progress,
+                )
+            except Exception as ex:
+                raise RuntimeError(
+                    f"Failed to download model artifact by reference from the given Object Storage path `{bucket_uri}`."
+                    f"See Exception: {ex}"
+                )
