@@ -46,7 +46,7 @@ class MetadataMixin:
     """
 
     def _populate_metadata_taxonomy(
-        self, model: callable = None, use_case_type: str = None
+        self, model: callable = None, use_case_type: str = None, **kwargs
     ):
         """Populates the taxonomy metadata.
 
@@ -211,6 +211,7 @@ class MetadataMixin:
         self,
         X_sample: Union[List, Tuple, pd.DataFrame, pd.Series, np.ndarray] = None,
         y_sample: Union[List, Tuple, pd.DataFrame, pd.Series, np.ndarray] = None,
+        **kwargs,
     ):
         """
         Any Framework-specific work before generic schema generation.
@@ -271,6 +272,7 @@ class MetadataMixin:
         ignore_pending_changes: bool = True,
         max_col_num: int = DATA_SCHEMA_MAX_COL_NUM,
         ignore_conda_error: bool = False,
+        **kwargs,
     ):
         """Populates input schema and output schema.
         If the schema exceeds the limit of 32kb, save as json files to the artifact directory.
@@ -312,7 +314,7 @@ class MetadataMixin:
             ].value
 
         self._populate_metadata_taxonomy(
-            model=self.estimator, use_case_type=use_case_type
+            model=self.estimator, use_case_type=use_case_type, **kwargs
         )
         if not ignore_conda_error:
             self._populate_metadata_custom()
@@ -321,6 +323,7 @@ class MetadataMixin:
             X_sample=X_sample,
             y_sample=y_sample,
             max_col_num=max_col_num,
+            **kwargs,
         )
         self._populate_provenance_metadata(
             training_script_path=training_script_path,
@@ -342,6 +345,7 @@ class MetadataMixin:
         X_sample: Union[List, Tuple, pd.DataFrame, pd.Series, np.ndarray] = None,
         y_sample: Union[List, Tuple, pd.DataFrame, pd.Series, np.ndarray] = None,
         max_col_num: int = DATA_SCHEMA_MAX_COL_NUM,
+        **kwargs,
     ):
         """Populate input and output schemas.
         If the schema exceeds the limit of 32kb, save as json files to the artifact dir.
@@ -365,16 +369,18 @@ class MetadataMixin:
             X_sample = data_sample.X
             y_sample = data_sample.y
 
-        X_sample, y_sample = self._prepare_data_for_schema(X_sample, y_sample)
+        X_sample, y_sample = self._prepare_data_for_schema(X_sample, y_sample, **kwargs)
         self.schema_input = self._populate_schema(
             X_sample,
             schema_file_name=INPUT_SCHEMA_FILE_NAME,
             max_col_num=max_col_num,
+            **kwargs,
         )
         self.schema_output = self._populate_schema(
             y_sample,
             schema_file_name=OUTPUT_SCHEMA_FILE_NAME,
             max_col_num=max_col_num,
+            **kwargs,
         )
 
     def _populate_schema(
@@ -382,6 +388,7 @@ class MetadataMixin:
         data: Union[list, tuple, pd.Series, np.ndarray, pd.DataFrame],
         schema_file_name: str,
         max_col_num: int,
+        **kwargs,
     ):
         """Populates schema and if the schema exceeds the limit of 32kb, save as a json file to the artifact_dir.
 
@@ -405,7 +412,10 @@ class MetadataMixin:
             if data is not None:
                 data = utils.to_dataframe(data)
                 schema = data.ads.model_schema(max_col_num=max_col_num)
-                schema.to_json_file(os.path.join(self.artifact_dir, schema_file_name))
+                schema.to_json_file(
+                    file_path=os.path.join(self.artifact_dir, schema_file_name),
+                    storage_options=kwargs.pop("auth", {}),
+                )
                 if self._validate_schema_size(schema, schema_file_name):
                     result = schema
         except DataSizeTooWide:
