@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8; -*-
 
-# Copyright (c) 2022, 2023 Oracle and/or its affiliates.
+# Copyright (c) 2022, 2024 Oracle and/or its affiliates.
 # Licensed under the Universal Permissive License v 1.0 as shown at https://oss.oracle.com/licenses/upl/
 
 import logging
@@ -17,7 +17,8 @@ from ads.common.oci_datascience import OCIDataScienceMixin
 from ads.common.oci_mixin import OCIWorkRequestMixin
 from ads.common.oci_resource import SEARCH_TYPE, OCIResource
 from ads.common.utils import extract_region
-from ads.model.deployment.model_deployer import ModelDeployer
+from ads.common.work_request import DataScienceWorkRequest
+from ads.model.deployment import ModelDeployment
 from oci.data_science.models import (
     ArtifactExportDetailsObjectStorage,
     ArtifactImportDetailsObjectStorage,
@@ -37,20 +38,22 @@ MODEL_NEEDS_TO_BE_SAVED = (
     "Model needs to be saved to the Model Catalog before it can be accessed."
 )
 
+MODEL_BY_REFERENCE_DESC = "modelDescription"
 
-class ModelProvenanceNotFoundError(Exception):
+
+class ModelProvenanceNotFoundError(Exception):  # pragma: no cover
     pass
 
 
-class ModelArtifactNotFoundError(Exception):
+class ModelArtifactNotFoundError(Exception):  # pragma: no cover
     pass
 
 
-class ModelNotSavedError(Exception):
+class ModelNotSavedError(Exception):  # pragma: no cover
     pass
 
 
-class ModelWithActiveDeploymentError(Exception):
+class ModelWithActiveDeploymentError(Exception):  # pragma: no cover
     pass
 
 
@@ -303,18 +306,25 @@ class OCIDataScienceModel(
     @check_for_model_id(
         msg="Model needs to be saved to the Model Catalog before the artifact can be created."
     )
-    def create_model_artifact(self, bytes_content: BytesIO) -> None:
+    def create_model_artifact(
+        self,
+        bytes_content: BytesIO,
+        extension: str = None,
+    ) -> None:
         """Creates model artifact for specified model.
 
         Parameters
         ----------
         bytes_content: BytesIO
             Model artifacts to upload.
+        extension: str
+            File extension, defaults to zip
         """
+        ext = ".json" if extension and extension.lower() == ".json" else ".zip"
         self.client.create_model_artifact(
             self.id,
             bytes_content,
-            content_disposition=f'attachment; filename="{self.id}.zip"',
+            content_disposition=f'attachment; filename="{self.id}{ext}"',
         )
 
     @check_for_model_id(
@@ -361,9 +371,8 @@ class OCIDataScienceModel(
             ).headers["opc-work-request-id"]
 
             # Show progress of importing artifacts
-            self._wait_for_work_request(
-                work_request_id=work_request_id,
-                num_steps=2,
+            DataScienceWorkRequest(work_request_id).wait_work_request(
+                progress_bar_description="Importing model artifacts."
             )
         except ServiceError as ex:
             if ex.status == 404:
@@ -408,9 +417,8 @@ class OCIDataScienceModel(
         ).headers["opc-work-request-id"]
 
         # Show progress of exporting model artifacts
-        self._wait_for_work_request(
-            work_request_id=work_request_id,
-            num_steps=3,
+        DataScienceWorkRequest(work_request_id).wait_work_request(
+            progress_bar_description="Exporting model artifacts."
         )
 
     @check_for_model_id(
@@ -424,10 +432,14 @@ class OCIDataScienceModel(
         OCIDataScienceModel
             The `OCIDataScienceModel` instance (self).
         """
+
+        model_details = self.to_oci_model(UpdateModelDetails)
+
+        # Clean up the model version set, otherwise it throws an error that model is already
+        # associated with the model version set.
+        model_details.model_version_set_id = None
         return self.update_from_oci_model(
-            self.client.update_model(
-                self.id, self.to_oci_model(UpdateModelDetails)
-            ).data
+            self.client.update_model(self.id, model_details).data
         )
 
     @check_for_model_id(
@@ -469,9 +481,7 @@ class OCIDataScienceModel(
                 logger.info(
                     f"Deleting model deployment `{oci_model_deployment.identifier}`."
                 )
-                ModelDeployer(ds_client=self.client).delete(
-                    model_deployment_id=oci_model_deployment.identifier
-                )
+                ModelDeployment.from_id(oci_model_deployment.identifier).delete()
 
         logger.info(f"Deleting model `{self.id}`.")
         self.client.delete_model(self.id)
@@ -543,58 +553,18 @@ class OCIDataScienceModel(
             raise ValueError("Model OCID not provided.")
         return super().from_ocid(ocid)
 
-    def _wait_for_work_request(self, work_request_id: str, num_steps: int = 3) -> None:
-        """Waits for the work request to be completed.
-
-        Parameters
-        ----------
-        work_request_id: str
-            Work Request OCID.
-        num_steps: (int, optional). Defaults to 3.
-            Number of steps for the progress indicator.
-
+    def is_model_by_reference(self):
+        """Checks if model is created by reference
         Returns
         -------
-        None
+            bool flag denoting whether model was created by reference.
+
         """
-        STOP_STATE = (
-            WorkRequest.STATUS_SUCCEEDED,
-            WorkRequest.STATUS_CANCELED,
-            WorkRequest.STATUS_FAILED,
-        )
-        work_request_logs = []
-
-        i = 0
-        with utils.get_progress_bar(num_steps) as progress:
-            while not work_request_logs or len(work_request_logs) < num_steps:
-                time.sleep(_REQUEST_INTERVAL_IN_SEC)
-                new_work_request_logs = []
-
-                try:
-                    work_request = self.client.get_work_request(work_request_id).data
-                    work_request_logs = self.client.list_work_request_logs(
-                        work_request_id
-                    ).data
-                except Exception as ex:
-                    logger.warn(ex)
-
-                new_work_request_logs = (
-                    work_request_logs[i:] if work_request_logs else []
-                )
-
-                for wr_item in new_work_request_logs:
-                    progress.update(wr_item.message)
-                    i += 1
-
-                if work_request and work_request.status in STOP_STATE:
-                    if work_request.status != WorkRequest.STATUS_SUCCEEDED:
-                        if new_work_request_logs:
-                            raise Exception(new_work_request_logs[-1].message)
-                        else:
-                            raise Exception(
-                                "Error occurred in attempt to perform the operation. "
-                                "Check the service logs to get more details. "
-                                f"{work_request}"
-                            )
-                    else:
-                        break
+        if self.custom_metadata_list:
+            for metadata in self.custom_metadata_list:
+                if (
+                    metadata.key == MODEL_BY_REFERENCE_DESC
+                    and metadata.value.lower() == "true"
+                ):
+                    return True
+        return False
