@@ -1,18 +1,15 @@
 #!/usr/bin/env python
 # -*- coding: utf-8; -*-
 
-# Copyright (c) 2023 Oracle and/or its affiliates.
+# Copyright (c) 2024 Oracle and/or its affiliates.
 # Licensed under the Universal Permissive License v 1.0 as shown at https://oss.oracle.com/licenses/upl/
 
 from functools import wraps
-import json
-import time
 import logging
 from typing import Callable, List
 from ads.common.oci_datascience import OCIDataScienceMixin
-from ads.common import utils as progress_bar_utils
+from ads.common.work_request import DataScienceWorkRequest
 from ads.config import PROJECT_OCID
-from ads.model.deployment.common import utils
 from ads.model.deployment.common.utils import OCIClientManager, State
 import oci
 
@@ -23,10 +20,6 @@ from oci.data_science.models import (
 
 DEFAULT_WAIT_TIME = 1200
 DEFAULT_POLL_INTERVAL = 10
-DEACTIVATE_WORKFLOW_STEPS = 2
-DELETE_WORKFLOW_STEPS = 2
-ACTIVATE_WORKFLOW_STEPS = 6
-CREATE_WORKFLOW_STEPS = 6
 ALLOWED_STATUS = [
     State.ACTIVE.name,
     State.CREATING.name,
@@ -74,11 +67,11 @@ def check_for_model_deployment_id(msg: str = MODEL_DEPLOYMENT_NEEDS_TO_BE_DEPLOY
     return decorator
 
 
-class MissingModelDeploymentIdError(Exception):
+class MissingModelDeploymentIdError(Exception):  # pragma: no cover
     pass
 
 
-class MissingModelDeploymentWorkflowIdError(Exception):
+class MissingModelDeploymentWorkflowIdError(Exception):  # pragma: no cover
     pass
 
 
@@ -180,36 +173,37 @@ class OCIDataScienceModelDeployment(
         OCIDataScienceModelDeployment
             The `OCIDataScienceModelDeployment` instance (self).
         """
-        logger.info(f"Activating model deployment `{self.id}`.")
-        response = self.client.activate_model_deployment(
-            self.id,
-        )
+        dsc_model_deployment = OCIDataScienceModelDeployment.from_id(self.id)
+        if dsc_model_deployment.lifecycle_state == self.LIFECYCLE_STATE_ACTIVE:
+            raise Exception(
+                f"Model deployment {dsc_model_deployment.id} is already in active state."
+            )
 
-        if wait_for_completion:
+        if dsc_model_deployment.lifecycle_state == self.LIFECYCLE_STATE_INACTIVE:
+            logger.info(f"Activating model deployment `{self.id}`.")
+            response = self.client.activate_model_deployment(
+                self.id,
+            )
 
-            self.workflow_req_id = response.headers.get("opc-work-request-id", None)
-            oci_model_deployment_object = self.client.get_model_deployment(self.id).data
-            current_state = State._from_str(oci_model_deployment_object.lifecycle_state)
-            model_deployment_id = self.id
+            if wait_for_completion:
+                self.workflow_req_id = response.headers.get("opc-work-request-id", None)
 
-            try:
-                self._wait_for_progress_completion(
-                    State.ACTIVE.name,
-                    ACTIVATE_WORKFLOW_STEPS,
-                    [State.FAILED.name, State.INACTIVE.name],
-                    self.workflow_req_id,
-                    current_state,
-                    model_deployment_id,
-                    max_wait_time,
-                    poll_interval,
-                )
-            except Exception as e:
-                logger.error(
-                    f"Error while trying to activate model deployment: {self.id}"
-                )
-                raise e
+                try:
+                    DataScienceWorkRequest(self.workflow_req_id).wait_work_request(
+                        progress_bar_description="Activating model deployment",
+                        max_wait_time=max_wait_time, 
+                        poll_interval=poll_interval
+                    )
+                except Exception as e:
+                    logger.error(
+                        "Error while trying to activate model deployment: " + str(e)
+                    )
 
-        return self.sync()
+            return self.sync()
+        else:
+            raise Exception(
+                f"Can't activate model deployment {dsc_model_deployment.id} when it's in {dsc_model_deployment.lifecycle_state} state."
+            )
 
     def create(
         self,
@@ -239,30 +233,19 @@ class OCIDataScienceModelDeployment(
         response = self.client.create_model_deployment(create_model_deployment_details)
         self.update_from_oci_model(response.data)
         logger.info(f"Creating model deployment `{self.id}`.")
+        print(f"Model Deployment OCID: {self.id}")
 
         if wait_for_completion:
-
             self.workflow_req_id = response.headers.get("opc-work-request-id", None)
-            res_payload = json.loads(str(response.data))
-            current_state = State._from_str(res_payload["lifecycle_state"])
-            model_deployment_id = self.id
 
             try:
-                self._wait_for_progress_completion(
-                    State.ACTIVE.name,
-                    CREATE_WORKFLOW_STEPS,
-                    [State.FAILED.name, State.INACTIVE.name],
-                    self.workflow_req_id,
-                    current_state,
-                    model_deployment_id,
-                    max_wait_time,
-                    poll_interval,
+                DataScienceWorkRequest(self.workflow_req_id).wait_work_request(
+                    progress_bar_description="Creating model deployment",
+                    max_wait_time=max_wait_time, 
+                    poll_interval=poll_interval
                 )
             except Exception as e:
-                logger.error(
-                    f"Error while trying to create model deployment: {self.id}"
-                )
-                raise e
+                logger.error("Error while trying to create model deployment: " + str(e))
 
         return self.sync()
 
@@ -293,36 +276,37 @@ class OCIDataScienceModelDeployment(
         OCIDataScienceModelDeployment
             The `OCIDataScienceModelDeployment` instance (self).
         """
-        logger.info(f"Deactivating model deployment `{self.id}`.")
-        response = self.client.deactivate_model_deployment(
-            self.id,
-        )
+        dsc_model_deployment = OCIDataScienceModelDeployment.from_id(self.id)
+        if dsc_model_deployment.lifecycle_state == self.LIFECYCLE_STATE_INACTIVE:
+            raise Exception(
+                f"Model deployment {dsc_model_deployment.id} is already in inactive state."
+            )
 
-        if wait_for_completion:
+        if dsc_model_deployment.lifecycle_state == self.LIFECYCLE_STATE_ACTIVE:
+            logger.info(f"Deactivating model deployment `{self.id}`.")
+            response = self.client.deactivate_model_deployment(
+                self.id,
+            )
 
-            self.workflow_req_id = response.headers.get("opc-work-request-id", None)
-            oci_model_deployment_object = self.client.get_model_deployment(self.id).data
-            current_state = State._from_str(oci_model_deployment_object.lifecycle_state)
-            model_deployment_id = self.id
+            if wait_for_completion:
+                self.workflow_req_id = response.headers.get("opc-work-request-id", None)
 
-            try:
-                self._wait_for_progress_completion(
-                    State.INACTIVE.name,
-                    DEACTIVATE_WORKFLOW_STEPS,
-                    [State.FAILED.name],
-                    self.workflow_req_id,
-                    current_state,
-                    model_deployment_id,
-                    max_wait_time,
-                    poll_interval,
-                )
-            except Exception as e:
-                logger.error(
-                    f"Error while trying to deactivate model deployment: {self.id}"
-                )
-                raise e
+                try:
+                    DataScienceWorkRequest(self.workflow_req_id).wait_work_request(
+                        progress_bar_description="Deactivating model deployment",
+                        max_wait_time=max_wait_time, 
+                        poll_interval=poll_interval
+                    )
+                except Exception as e:
+                    logger.error(
+                        "Error while trying to deactivate model deployment: " + str(e)
+                    )
 
-        return self.sync()
+            return self.sync()
+        else:
+            raise Exception(
+                f"Can't deactivate model deployment {dsc_model_deployment.id} when it's in {dsc_model_deployment.lifecycle_state} state."
+            )
 
     @check_for_model_deployment_id(
         msg="Model deployment needs to be deployed before it can be deleted."
@@ -351,34 +335,38 @@ class OCIDataScienceModelDeployment(
         OCIDataScienceModelDeployment
             The `OCIDataScienceModelDeployment` instance (self).
         """
+        dsc_model_deployment = OCIDataScienceModelDeployment.from_id(self.id)
+        if dsc_model_deployment.lifecycle_state in [
+            self.LIFECYCLE_STATE_DELETED,
+            self.LIFECYCLE_STATE_DELETING,
+        ]:
+            raise Exception(
+                f"Model deployment {dsc_model_deployment.id} is either deleted or being deleted."
+            )
+        if dsc_model_deployment.lifecycle_state not in [
+            self.LIFECYCLE_STATE_ACTIVE,
+            self.LIFECYCLE_STATE_FAILED,
+            self.LIFECYCLE_STATE_INACTIVE,
+        ]:
+            raise Exception(
+                f"Can't delete model deployment {dsc_model_deployment.id} when it's in {dsc_model_deployment.lifecycle_state} state."
+            )
         logger.info(f"Deleting model deployment `{self.id}`.")
         response = self.client.delete_model_deployment(
             self.id,
         )
 
         if wait_for_completion:
-
             self.workflow_req_id = response.headers.get("opc-work-request-id", None)
-            oci_model_deployment_object = self.client.get_model_deployment(self.id).data
-            current_state = State._from_str(oci_model_deployment_object.lifecycle_state)
-            model_deployment_id = self.id
 
             try:
-                self._wait_for_progress_completion(
-                    State.DELETED.name,
-                    DELETE_WORKFLOW_STEPS,
-                    [State.FAILED.name, State.INACTIVE.name],
-                    self.workflow_req_id,
-                    current_state,
-                    model_deployment_id,
-                    max_wait_time,
-                    poll_interval,
+                DataScienceWorkRequest(self.workflow_req_id).wait_work_request(
+                    progress_bar_description="Deleting model deployment",
+                    max_wait_time=max_wait_time, 
+                    poll_interval=poll_interval
                 )
             except Exception as e:
-                logger.error(
-                    f"Error while trying to delete model deployment: {self.id}"
-                )
-                raise e
+                logger.error("Error while trying to delete model deployment: " + str(e))
 
         return self.sync()
 
@@ -432,8 +420,7 @@ class OCIDataScienceModelDeployment(
             )
             self.workflow_req_id = response.headers.get("opc-work-request-id", None)
         except Exception as e:
-            logger.error(f"Error while trying to update model deployment: {self.id}")
-            raise e
+            logger.error("Error while trying to update model deployment: " + str(e))
 
         return self.sync()
 
@@ -511,90 +498,3 @@ class OCIDataScienceModelDeployment(
             An instance of `OCIDataScienceModelDeployment`.
         """
         return super().from_ocid(model_deployment_id)
-
-    def _wait_for_progress_completion(
-        self,
-        final_state: str,
-        work_flow_step: int,
-        disallowed_final_states: List[str],
-        work_flow_request_id: str,
-        state: State,
-        model_deployment_id: str,
-        max_wait_time: int = DEFAULT_WAIT_TIME,
-        poll_interval: int = DEFAULT_POLL_INTERVAL,
-    ):
-        """_wait_for_progress_completion blocks until progress is completed.
-
-        Parameters
-        ----------
-        final_state: str
-            Final state of model deployment aimed to be reached.
-        work_flow_step: int
-            Number of work flow step of the request.
-        disallowed_final_states: list[str]
-            List of disallowed final state to be reached.
-        work_flow_request_id: str
-            The id of work flow request.
-        state: State
-            The current state of model deployment.
-        model_deployment_id: str
-            The ocid of model deployment.
-        max_wait_time: int
-            Maximum amount of time to wait in seconds (Defaults to 1200).
-            Negative implies infinite wait time.
-        poll_interval: int
-            Poll interval in seconds (Defaults to 10).
-        """
-
-        start_time = time.time()
-        prev_message = ""
-        prev_workflow_stage_len = 0
-        current_state = state or State.UNKNOWN
-        with progress_bar_utils.get_progress_bar(work_flow_step) as progress:
-            if max_wait_time > 0 and utils.seconds_since(start_time) >= max_wait_time:
-                utils.get_logger().error(
-                    f"Max wait time ({max_wait_time} seconds) exceeded."
-                )
-            while (
-                max_wait_time < 0 or utils.seconds_since(start_time) < max_wait_time
-            ) and current_state.name.upper() != final_state:
-                if current_state.name.upper() in disallowed_final_states:
-                    utils.get_logger().info(
-                        f"Operation failed due to deployment reaching state {current_state.name.upper()}. Use Deployment ID for further steps."
-                    )
-                    break
-
-                prev_state = current_state.name
-                try:
-                    model_deployment_payload = json.loads(
-                        str(self.client.get_model_deployment(model_deployment_id).data)
-                    )
-                    current_state = (
-                        State._from_str(model_deployment_payload["lifecycle_state"])
-                        if "lifecycle_state" in model_deployment_payload
-                        else State.UNKNOWN
-                    )
-                    workflow_payload = self.client.list_work_request_logs(
-                        work_flow_request_id
-                    ).data
-                    if isinstance(workflow_payload, list) and len(workflow_payload) > 0:
-                        if prev_message != workflow_payload[-1].message:
-                            for _ in range(
-                                len(workflow_payload) - prev_workflow_stage_len
-                            ):
-                                progress.update(workflow_payload[-1].message)
-                            prev_workflow_stage_len = len(workflow_payload)
-                            prev_message = workflow_payload[-1].message
-                            prev_workflow_stage_len = len(workflow_payload)
-                    if prev_state != current_state.name:
-                        utils.get_logger().info(
-                            f"Status Update: {current_state.name} in {utils.seconds_since(start_time)} seconds"
-                        )
-                except Exception as e:
-                    # utils.get_logger().warning(
-                    #     "Unable to update deployment status. Details: %s", format(
-                    #         e)
-                    # )
-                    pass
-                time.sleep(poll_interval)
-            progress.update("Done")
