@@ -16,14 +16,17 @@ import stat
 import conda_pack
 
 import yaml
+import argparse
 
 
-def main(pack_folder_path):
+def main(pack_folder_path, manifest_file=None):
     slug = os.path.basename(pack_folder_path)
-    manifest_path = glob.glob(os.path.join(pack_folder_path, "*_manifest.yaml"))[0]
+    manifest_path = (
+        manifest_file or glob.glob(os.path.join(pack_folder_path, "*_manifest.yaml"))[0]
+    )
     with open(manifest_path) as f:
         env = yaml.safe_load(f.read())
-
+    
     with tempfile.TemporaryDirectory() as td:
         process = subprocess.Popen(
             ["conda", "env", "export", "--prefix", pack_folder_path],
@@ -31,7 +34,8 @@ def main(pack_folder_path):
             stderr=subprocess.PIPE,
         )
         stdout, stderr = process.communicate()
-        if stderr:
+        
+        if process.returncode and stderr:
             print(stderr)
             raise Exception(
                 f"Error export environment information from {pack_folder_path}"
@@ -59,8 +63,10 @@ def main(pack_folder_path):
             raise RuntimeError(
                 "Error creating the pack file using `conda_pack.pack()`."
             )
+        print(f"Copy {pack_file} to {pack_folder_path}")
         shutil.copy(pack_file, pack_folder_path)
         file_path = os.path.join(pack_folder_path, os.path.basename(pack_file))
+        print(f"Pack built at {file_path}")
         print(
             f"changing permission for {file_path}",
             flush=True,
@@ -69,4 +75,14 @@ def main(pack_folder_path):
 
 
 if __name__ == "__main__":
-    main(sys.argv[1])
+    parser = argparse.ArgumentParser(
+        prog="Prepare conda archive",
+        description="Uses conda_pack library to pack the conda environment.",
+    )
+    parser.add_argument("--conda-path", type=str, help="Path to the conda environment")
+    parser.add_argument(
+        "--manifest-location", type=str, default=None, help="Path to manifest location"
+    )
+    args = parser.parse_args()
+
+    main(args.conda_path, args.manifest_location)
