@@ -5,5 +5,5 @@
 # Licensed under the Universal Permissive License v 1.0 as shown at https://oss.oracle.com/licenses/upl/
 
 
-class RequirementException(Exception):
+class RequirementException(Exception):   # pragma: no cover
     pass
